/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdio.h>
#include <mntent.h>

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>
#include <libxfce4panel/xfce-panel-plugin.h>

#define MNT "/proc/mounts"

enum /* persistence */
{
    LPP_ERROR = 0,
    LPP_HOME,
    LPP_ALL,
    LPP_NONE,
};

const gchar *icons[] = {
    "important",
    "user-home",
    "gnome-fs-server",
    "error"
};

const gchar *tips[] = {
    "Unable to find the status.",
    "User dir persistence only.",
    "Full persistence.",
    "Lock mode."
};

typedef struct
{
    XfcePanelPlugin *plugin;

    GtkWidget *ebox;
    GtkWidget *img;

    guint  persistence;
    gchar *device;
} LPlugin;

/* Linutop info */
static void
get_infos (LPlugin *lp)
{
    FILE *fp;
    struct mntent *fs;
    gboolean have_home = FALSE;
    gboolean have_cow = FALSE;
    gboolean have_rofsbis = FALSE;

    fp = setmntent (MNT, "r");
    if (G_UNLIKELY (!fp))
        return;

    while ((fs = getmntent (fp)) != NULL)
    {
        if (strncmp (fs->mnt_dir, "/cdrom", 6) == 0)
            lp->device = g_strndup (fs->mnt_fsname, 8); /* /dev/[hs]d? */
        if (strncmp (fs->mnt_dir, "/home", 5) == 0)
            have_home = TRUE;
        if (strncmp (fs->mnt_dir, "/cow", 4) == 0)
            have_cow = TRUE;
        if (strncmp (fs->mnt_dir, "/rofsbis", 8) == 0)
            have_rofsbis = TRUE;
    }
    endmntent (fp);

    lp->persistence = LPP_ERROR;
    if (have_home)
        lp->persistence = LPP_HOME;
    else if (have_cow)
    {
        if (have_rofsbis)
            lp->persistence = LPP_NONE; /* lock mode */
        else
            lp->persistence = LPP_ALL;
    }
}

/* UI */
LPlugin *
lp_new (XfcePanelPlugin *plugin)
{
    int        size;
    gchar     *tooltip;
    GdkPixbuf *icon;

    LPlugin *lp = panel_slice_new0 (LPlugin);
    lp->plugin = plugin;
    lp->device = NULL;
    get_infos (lp);

    size = xfce_panel_plugin_get_size (lp->plugin) - 6;
    icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
                icons[lp->persistence], size, 0, NULL);
    lp->img = gtk_image_new_from_pixbuf (icon);
    g_object_unref (G_OBJECT (icon));

    tooltip = g_strdup (tips[lp->persistence]);
    if (lp->device)
        tooltip = g_strconcat (tooltip, "\nThe root device is ",
                    lp->device, ".", NULL);

    gtk_widget_set_tooltip_text (lp->img,
                                   tooltip);
    g_free (tooltip);

    lp->ebox = gtk_event_box_new ();
    gtk_container_add (GTK_CONTAINER (lp->ebox), lp->img);

    gtk_widget_show_all (lp->ebox);

    return lp;
}

static void
lp_free (XfcePanelPlugin *plugin,
         LPlugin *lp)
{
    gtk_widget_destroy (lp->img);
    gtk_widget_destroy (lp->ebox);
    panel_slice_free (LPlugin, lp);
}

static void
lp_construct (XfcePanelPlugin *plugin)
{
    LPlugin *lp;

    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    lp = lp_new (plugin);
    gtk_container_add (GTK_CONTAINER (plugin), lp->ebox);
    xfce_panel_plugin_add_action_widget (plugin, lp->ebox);

    g_signal_connect (G_OBJECT (plugin), "free-data",
        G_CALLBACK (lp_free), lp);
}

XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (lp_construct);


