/* main.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <pwd.h>

#include <glib.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>

#include <libxfcegui4/libxfcegui4.h>

#include "main.h"

#define REBOOTCMD "shutdown -r now"

static gchar *resolutions[11] = {
  "Auto",
  "640x480 (15'')",
  "800x600 (15'')",
  "1024x768 (15'')",
  "1280x1024 (17-19'')",
  "1280x768 (17'' wide)",
  "1366x768 (19'' wide)",
  "1600x1200 (20'' wide)",
  "1680x1050 (22'' wide)",
  "1920x1440 (Max)",
  NULL
};

const gchar *default_syscfg =
  "DEFAULT vesamenu.c32\n"
  "PROMPT 0\n"
  "MENU BACKGROUND splash.jpg\n"
  "MENU COLOR sel 7;31;40 #900090f0 #00000000 std\n"
  "MENU COLOR unsel 31;40 #90000000 #00000000 std\n"
  "MENU COLOR tabmsg 31;40 #900090f0 #00000000 std\n"
  "MENU WIDTH 66\n"
  "MENU VSHIFT 20\n"
  "MENU ROWS 4\n"
  "MENU AUTOBOOT\n"
  "MENU TABMSG\n"
  "LABEL linutop\n"
  "MENU label Start Linutop\n"
  "KERNEL vmlinuz\n"
  "append  boot=lrd initrd=initrd.gz ramdisk_size=1048576 root=/dev/ram "
  "rw quiet splash -- "
  "%s\n"
  "LABEL failsafe\n"
  "MENU label Start Linutop in failsafe mode\n"
  "KERNEL vmlinuz\n"
  "append  boot=lrd initrd=initrd.gz ramdisk_size=1048576 root=/dev/ram rw "
  "quiet splash -- checkfs persistent=none\n"
  "LABEL pxe\n"
  "MENU label PXE Boot\n"
  "KERNEL rtl8139.zli\n"
  "LABEL restore\n"
  "MENU hide\n"
  "KERNEL vmlinuz\n"
  "append  boot=lrd initrd=initrd.gz ramdisk_size=1048576 root=/dev/ram rw "
  "quiet splash -- restore=true\n"
  "LABEL update\n"
  "MENU label Linutop update\n"
  "KERNEL vmup\n"
  "append  boot=update initrd=update.gz ramdisk_size=1048576 root=/dev/ram rw\n"
  "TIMEOUT 10\n";

const gchar *locked_syscfg =
  "DEFAULT vesamenu.c32\n"
  "PROMPT 0\n"
  "MENU BACKGROUND splash.jpg\n"
  "MENU COLOR sel 7;31;40 #900090f0 #00000000 std\n"
  "MENU COLOR unsel 31;40 #90000000 #00000000 std\n"
  "MENU COLOR tabmsg 31;40 #900090f0 #00000000 std\n"
  "MENU WIDTH 66\n"
  "MENU VSHIFT 20\n"
  "MENU ROWS 4\n"
  "MENU AUTOBOOT\n"
  "MENU TABMSG\n"
  "MENU MASTER PASSWD %s\n"
  "LABEL linutop\n"
  "MENU label Start Linutop\n"
  "KERNEL vmlinuz\n"
  "append  boot=lrd initrd=initrd.gz ramdisk_size=1048576 root=/dev/ram "
  "rw quiet splash -- " "%s\n" "TIMEOUT 1\n";

struct InitialConf {
    gchar *tz_txt;
    gchar *locale_txt;
    gchar *kbd_code_txt;
    gchar *screen_txt;
    gchar *persistent;
    gboolean lock;
} *initial_conf;

gboolean reboot_needed = FALSE;

gchar *
get_home ()
{
  struct passwd *pw;
  uid_t uid = 0;
  char *tmp;

  tmp = getenv ("SUDO_UID");
  if (tmp)
    uid = atoi (tmp);
  tmp = "";
  if (uid >= 0)
    pw = getpwuid (uid);
  if (pw)
    tmp = pw->pw_dir;

  return tmp;
}

gchar *
get_first_word (gchar * line)
{
  gchar **tab;
  gchar *word;
  const char *del = " ";
  if (strncmp (line, "Auto", 4) == 0)
      return NULL;
  tab = g_strsplit (line, del, 2);
  if (tab[0] != '\0')
    {
      word = g_strdup (tab[0]);
      g_strfreev (tab);
      return word;
    }
  return NULL;
}

static void
run_user_cmd (const char *user_cmd, gboolean sn)
{
  uid_t uid = 0;
  char *cuid = NULL;
  gchar cmd[100];
  gchar *run_sn = sn ? "" : "-n";

  cuid = getenv ("SUDO_UID");
  if (cuid)
    uid = atoi (cuid);
  if (uid >= 0)
    {
      g_sprintf (cmd, "lstart %s -c \"%s\" -u %d", run_sn, user_cmd, uid);
      xfce_exec (cmd, FALSE, FALSE, NULL);
    }
}

static void
run_help (GtkWidget * widget, gpointer data)
{
  run_user_cmd ("mousepad /cdrom/readme.txt", TRUE);
}

void
apply_kbd_config (gchar * kbd)
{
  gchar cmd[100];
  g_sprintf (cmd, "setxkbmap %s", kbd);
  xfce_exec (cmd, FALSE, FALSE, NULL);
}

static void
delete_event (GtkWidget * widget, GdkEvent * event, gpointer data)
{
  /* restore the initial kbd conf */
  apply_kbd_config (initial_conf->kbd_code_txt);
  gtk_main_quit ();
}


static void
kbd_btn_clicked (GtkWidget * widget, gpointer data)
{
  gchar *str = layout_dialog_run ();
  if (str)
    {
      dialog->kbd_code_txt = str;
      gtk_entry_set_text (GTK_ENTRY (dialog->kbd_entry),
			  (const char *) (get_kbd_name (str)));
    }
}

static void
tz_btn_clicked (GtkWidget * widget, gpointer data)
{
  gchar *str = tz_dialog_run ();
  if (str)
    gtk_entry_set_text (GTK_ENTRY (dialog->tz_entry), str);
}

static void
locale_btn_clicked (GtkWidget * widget, gpointer data)
{
  gchar *str = locale_dialog_run ();
  if (str)
    gtk_entry_set_text (GTK_ENTRY (dialog->locale_entry), str);
}

/* */
const gchar *
get_text (const gchar * locale, const gchar * kbd,
	  const gchar * tz, const gchar * persist)
{
  dialog->screen_txt = get_first_word (gtk_combo_box_get_active_text
		    (GTK_COMBO_BOX (dialog->screen_combo)));

  gchar *nox =
    (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->nox_chk)) ==
     TRUE) ? " noX" : "";
  gchar *checkfs =
    (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkfs_chk)) ==
     TRUE) ? " checkfs" : "";

  const gchar *datas =
    g_strconcat (strcmp (locale, "") !=
		 0 ? g_strconcat ("locale=", locale, NULL) : "",
		 strcmp (kbd,
			 "") != 0 ? g_strconcat (" console-setup/layoutcode=",
						 kbd,
						 " xkbdlayout=", kbd,
						 NULL) : "",
		 strcmp (tz, "") != 0 ? g_strconcat (" timezone=", tz,
						     NULL) : "",
		 dialog->screen_txt ? g_strconcat (" screensize=",
                         dialog->screen_txt, NULL) : "",
		 nox,
		 checkfs,
		 dialog->lock == TRUE ? " lock" : "",
		 strcmp (persist, "") != 0 ? g_strconcat (" persistent=",
							  persist, NULL) : "",
         dialog->rootdevice ? g_strconcat (" rootdevice=",
                             dialog->rootdevice, NULL) : "",
		 NULL);
  gchar str[BUFSIZ];
  if (dialog->lock)
    g_sprintf (str, locked_syscfg, dialog->lock_pwd, datas);
  else
    g_sprintf (str, default_syscfg, datas);

  return g_strdup (str);
}

static void
update_text (GtkWidget * widget, gpointer data)
{
  const gchar *locale = dialog->locale_txt;
  const gchar *kbd = dialog->kbd_code_txt;
  const gchar *tz = gtk_entry_get_text (GTK_ENTRY (dialog->tz_entry));
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk)))
    dialog->persistent = "full";
  else
    dialog->persistent = "";

  const gchar *str = g_strdup (get_text (locale, kbd, tz, dialog->persistent));
  dialog->output = g_strdup (str);

#ifdef HAVE_VIEW
  GtkTextBuffer *buff =
    gtk_text_view_get_buffer (GTK_TEXT_VIEW (dialog->text));
  gtk_text_buffer_set_text (buff, str, -1);
#endif /* HAVE_VIEW */
}

static void
net_admin (GtkWidget * widget, gpointer data)
{
  xfce_exec ("network-admin", FALSE, TRUE, NULL);
}

static void
appstarter (GtkWidget * widget, gpointer data)
{
  run_user_cmd ("/usr/bin/appstarter", TRUE);
}

static void
lbackup (GtkWidget * widget, gpointer data)
{
  xfce_exec ("/usr/bin/lbackup", FALSE, TRUE, NULL);
}

static void
usb_format (GtkWidget * widget, gpointer data)
{
    xfce_exec ("lusbclean", FALSE, TRUE, NULL);
}

static void
change_pwd (GtkWidget * w, gpointer data)
{
  gchar *tmp = get_lock_passwd ();
  if (tmp)
    dialog->lock_pwd = tmp;
  update_text (NULL, NULL);
}

static void
lock_toggled (GtkWidget * b, gpointer data)
{
  gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b));

  if (active)
    {
      dialog->lock = TRUE;
      if (!dialog->lock_pwd)
	{
	  gchar *tmp = get_lock_passwd ();
	  if (tmp)
	    {
	      gtk_widget_set_sensitive (dialog->vbox_lock, TRUE);
	      dialog->lock_pwd = tmp;
	    }
	  else
	    {
	      gtk_widget_set_sensitive (dialog->vbox_lock, FALSE);
	      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), FALSE);
	    }
	}
      else
	gtk_widget_set_sensitive (dialog->vbox_lock, TRUE);
    }
  else
  {
    dialog->lock = FALSE;
    gtk_widget_set_sensitive (dialog->vbox_lock, FALSE);
  }

  update_text (NULL, NULL);
}

static void
set_default_resolution (void)
{
  int i = 1;
  while (resolutions[i] != '\0')
    {
      if (strcmp (dialog->screen_txt, get_first_word (resolutions[i])) == 0)
	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), i);
      i++;
    }
}

static void
read_config ()
{
  FILE *fp;
  gchar nexttime[64];

  dialog->tz_txt = "Europe/London";
  dialog->locale_txt = "en_US.UTF-8";
  dialog->kbd_code_txt = "us";
  dialog->screen_txt = "";
  dialog->persistent = "";
  dialog->lock_pwd = NULL;
  dialog->lock = FALSE;
  dialog->rootdevice = NULL;

  if (G_LIKELY ((fp = fopen (conf_file, "r"))))
    {
      gchar *gline = NULL, *lline = NULL;
      gchar *tmp;
      size_t len = 0;
      ssize_t read;

      while ((read = getline (&tmp, &len, fp)) != -1)
	  {
	    if (!gline && (strncmp (tmp, "append  ", 8) == 0))
	    {
	      gline = g_strdup (tmp);
	      if (gline[strlen (gline) - 1] == '\n')
		    gline[strlen (gline) - 1] = '\0';
	    }
        if (!lline && (strncmp (tmp, "MENU MASTER PASSWD", 18) == 0))
        {
            dialog->lock = TRUE;
            lline = g_strdup (tmp);
            if (lline[strlen (lline) - 1] == '\n')
                lline[strlen (lline) - 1] = '\0';
            gchar **exploded = g_strsplit (lline, " ", 0);
            if (g_strv_length (exploded) == 4)
                dialog->lock_pwd = g_strdup (exploded[3]);
            g_strfreev (exploded);
        }
	  }
      g_free (lline);
      g_free (tmp);
      fclose (fp);

      if (G_LIKELY (gline))
	{
	  gchar **exploded = g_strsplit (gline, " ", 0);
	  int count = 0;
	  while (exploded[count] != '\0')
      {
	    gchar **item = g_strsplit (exploded[count], "=", 0);
	    if (g_strv_length (item) == 2)
		{
		  if (item[0] && item[1]
		      && strncmp (item[0], "xkbdlayout", 10) == 0)
		    dialog->kbd_code_txt = g_strdup (item[1]);
		  if (item[0] && item[1]
		      && strncmp (item[0], "locale", 6) == 0)
		    dialog->locale_txt = g_strdup (item[1]);
		  if (item[0] && item[1]
		      && strncmp (item[0], "timezone", 8) == 0)
		    dialog->tz_txt = g_strdup (item[1]);
		  if (item[0] && item[1]
		      && strncmp (item[0], "screensize", 10) == 0)
		    dialog->screen_txt = g_strdup (item[1]);
		  if (item[0] && item[1]
		      && strncmp (item[0], "persistent", 10) == 0)
		    dialog->persistent = g_strdup (item[1]);
		  if (item[0] && item[1]
		      && strncmp (item[0], "rootdevice", 10) == 0)
		    dialog->rootdevice = g_strdup (item[1]);
  	    }
        else
        {
		  if (strncmp (exploded[count], "noX", 3) == 0)
		    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
						  (dialog->nox_chk), TRUE);
		  if (strncmp (exploded[count], "checkfs", 3) == 0)
		    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
						  (dialog->checkfs_chk),
						  TRUE);
		  if (strncmp (exploded[count], "lockmode", 3) == 0)
		    {
		      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
						    (dialog->lock_chk),
						    TRUE);
		      gtk_widget_set_sensitive (dialog->vbox_lock, TRUE);
		    }
		}
        g_strfreev (item);
	    count++;
      }
	  g_free (gline);
	  g_strfreev (exploded);
	}
    }
  gtk_entry_set_text (GTK_ENTRY (dialog->kbd_entry),
		      (const char *) (get_kbd_name (dialog->kbd_code_txt)));
  gtk_entry_set_text (GTK_ENTRY (dialog->locale_entry),
		      get_locale_name (dialog->locale_txt));
  gtk_entry_set_text (GTK_ENTRY (dialog->tz_entry), dialog->tz_txt);
  if (strcmp (dialog->persistent, "full") == 0)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->full_chk), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->home_chk), TRUE);

  g_sprintf (nexttime, "%s/.config/autostart/lconfeditor.desktop",
	     get_home ());
  if ((g_access ("/cdrom/lconfeditor-stamp", R_OK) != 0)
      || (g_access (nexttime, R_OK) == 0))
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->next_time),
				    TRUE);
    }
  else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->next_time),
				    FALSE);
    }

  set_default_resolution ();

  initial_conf = g_new0 (struct InitialConf, 1);
  initial_conf->tz_txt = g_strdup (dialog->tz_txt);
  initial_conf->locale_txt = g_strdup (dialog->locale_txt);
  initial_conf->kbd_code_txt = g_strdup (dialog->kbd_code_txt);
  initial_conf->screen_txt = g_strdup (dialog->screen_txt);
  initial_conf->persistent = g_strdup (dialog->persistent);
  initial_conf->lock = dialog->lock;
}

static void
change_next_time ()
{
  gchar desktop_file[] = "/usr/share/applications/lconfeditor.desktop";
  gchar dir[64];
  gchar cmd[128];
  gchar *tmp;
  uid_t uid = 1000;

  tmp = getenv ("SUDO_UID");
  if (tmp)
    uid = atoi (tmp);
  g_sprintf (dir, "%s/.config/autostart/", get_home ());
  mkdir (dir, 0755);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->next_time)))
    {
      g_sprintf (cmd, "cp %s %s", desktop_file, dir);
      system ((const char *) cmd);
      g_sprintf (cmd, "%s/lconfeditor.desktop", dir);
      chown (dir, uid, uid);
    }
  else
    {
      g_sprintf (cmd, "%s/lconfeditor.desktop", dir);
      unlink (cmd);
    }
}

static void
save_config (GtkWidget * widget, gpointer data)
{
  FILE *f;

  if ((f = fopen (conf_file, "w")))
    {
      fputs (dialog->output, f);
      fclose (f);

      if (G_LIKELY ((f = fopen ("/cdrom/lconfeditor-stamp", "w"))))
	fclose (f);
    }
  change_next_time ();
  mount_save ();

  /* apply smb mounts */
  run_user_cmd ("linutop-startup --samba", FALSE);

  /* destroy the main window */
  gtk_widget_destroy (dialog->dialog);

  /* prompt the user for reboot if needed */
  if (strcmp (dialog->persistent, initial_conf->persistent) ||
     (dialog->lock != initial_conf->lock) ||
     strcmp (dialog->screen_txt, initial_conf->screen_txt))
        reboot_needed = warn_about_changes ();

  gtk_main_quit ();
}

static void
populate_screen_combo (GtkWidget * combo)
{
  int i = 0;
  while (resolutions[i] != '\0')
    {
      gtk_combo_box_append_text (GTK_COMBO_BOX (combo), resolutions[i]);
      i++;
    }
}

static void
create_dialog ()
{
  GtkWidget *label, *table, *vbox, *hbox, *notebook, *vbox2;
  GtkWidget *asrun_btn, *backup_btn, *format_btn, *image, *frame;
  GtkWidget *advanced;
  GtkWidget *mnt_hbox;
  GtkWidget *lock_w, *hr;
  GtkTooltips *tooltip;
#ifdef HAVE_VIEW
  GtkWidget *scwindow;
#endif

  gtk_window_set_default_icon_name ("preferences-desktop");

  dialog->dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog->dialog),
			_("Linutop Setup"));
  gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
  gtk_window_set_position (GTK_WINDOW (dialog->dialog), GTK_WIN_POS_CENTER);

  notebook = gtk_notebook_new ();
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->dialog)->vbox),
		     notebook);
  gtk_widget_show (notebook);

  /*
   * General tab
   */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);

  label = gtk_label_new (_("General"));
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

  table = gtk_table_new (3, 3, FALSE);
  gtk_widget_show (table);

  /* local settings */
  frame = xfce_framebox_new (_("Local settings:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), table);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
  gtk_widget_show (frame);

  label = gtk_label_new (_("Language: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL,
		    GTK_SHRINK, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  dialog->locale_entry = gtk_entry_new ();
  gtk_editable_set_editable (GTK_EDITABLE (dialog->locale_entry), FALSE);
  gtk_table_attach (GTK_TABLE (table), dialog->locale_entry, 1, 2, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (dialog->locale_entry);
  dialog->locale_btn = gtk_button_new ();
  gtk_table_attach (GTK_TABLE (table), dialog->locale_btn, 2, 3, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  image =
    gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
  gtk_container_add (GTK_CONTAINER (dialog->locale_btn), image);
  gtk_widget_show (image);
  tooltip = gtk_tooltips_new ();
  gtk_tooltips_set_tip (tooltip, dialog->locale_btn,
			_("Clic here to select your language"), "");
  gtk_widget_show (dialog->locale_btn);

  label = gtk_label_new (_("Keyboard layout: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL,
		    GTK_SHRINK, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  dialog->kbd_entry = gtk_entry_new ();
  gtk_editable_set_editable (GTK_EDITABLE (dialog->kbd_entry), FALSE);
  gtk_table_attach (GTK_TABLE (table), dialog->kbd_entry, 1, 2, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (dialog->kbd_entry);
  dialog->kbd_btn = gtk_button_new ();
  gtk_table_attach (GTK_TABLE (table), dialog->kbd_btn, 2, 3, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  image =
    gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
  gtk_container_add (GTK_CONTAINER (dialog->kbd_btn), image);
  gtk_widget_show (image);
  tooltip = gtk_tooltips_new ();
  gtk_tooltips_set_tip (tooltip, dialog->kbd_btn,
			_("Clic here to select your keyboard layout"), "");
  gtk_widget_show (dialog->kbd_btn);

  label = gtk_label_new (_("Timezone: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL,
		    GTK_SHRINK, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  dialog->tz_entry = gtk_entry_new ();
  gtk_editable_set_editable (GTK_EDITABLE (dialog->tz_entry), FALSE);
  gtk_table_attach (GTK_TABLE (table), dialog->tz_entry, 1, 2, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (dialog->tz_entry);
  dialog->tz_btn = gtk_button_new ();
  gtk_table_attach (GTK_TABLE (table), dialog->tz_btn, 2, 3, 2, 3, GTK_SHRINK,
		    GTK_SHRINK, 0, 0);
  image =
    gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
  gtk_container_add (GTK_CONTAINER (dialog->tz_btn), image);
  gtk_widget_show (image);
  tooltip = gtk_tooltips_new ();
  gtk_tooltips_set_tip (tooltip, dialog->tz_btn,
			_("Clic here to select your time zone"), "");
  gtk_widget_show (dialog->tz_btn);

  /* Display settings */
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  label = gtk_label_new (_("Screen size:"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);

  dialog->screen_combo = gtk_combo_box_new_text ();
  gtk_box_pack_start (GTK_BOX (hbox), dialog->screen_combo, TRUE, TRUE, 2);
  populate_screen_combo (dialog->screen_combo);
  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), 0);
  gtk_widget_show (dialog->screen_combo);

  frame = xfce_framebox_new (_("Display settings:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), hbox);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /* Autostarted app */
  vbox2 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);

  image =
    gtk_image_new_from_icon_name ("xfce4-autostart-editor",
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 2);
  gtk_widget_show (image);

  label = gtk_label_new (_("Configure autostarted applications"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  asrun_btn = gtk_button_new ();
  gtk_widget_set_sensitive (asrun_btn,
			    (g_access ("/usr/bin/appstarter", X_OK) == 0));
  gtk_container_add (GTK_CONTAINER (asrun_btn), hbox);
  gtk_widget_show (asrun_btn);

  gtk_box_pack_start (GTK_BOX (vbox2), asrun_btn, FALSE, FALSE, 2);
  gtk_widget_show (asrun_btn);

  dialog->next_time =
    gtk_check_button_new_with_label (_("Show this dialog at each boot"));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->next_time, FALSE, FALSE, 2);
  gtk_widget_show (dialog->next_time);

  frame = xfce_framebox_new (_("Autostarted application:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  label = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label), _
			("<span style=\"italic\" size=\"small\">most changes will be effective after reboot.</span>"));
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 4);
  gtk_widget_show (label);

  /*
   * Network tab
   */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);

  label = gtk_label_new (_("Network"));
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

  /* Network settings */
  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);

  image =
    gtk_image_new_from_icon_name ("network", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 2);
  gtk_widget_show (image);

  label = gtk_label_new (_("Configure your network"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  dialog->net_btn = gtk_button_new ();
  gtk_widget_set_sensitive (dialog->net_btn,
			    (g_access ("/usr/bin/network-admin", X_OK) == 0));
  gtk_container_add (GTK_CONTAINER (dialog->net_btn), hbox);
  gtk_widget_show (dialog->net_btn);

  frame = xfce_framebox_new (_("Network settings:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), dialog->net_btn);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /* samba */
  mnt_hbox = create_mount_gui ();
  frame = xfce_framebox_new (_("Windows shares:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), mnt_hbox);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /*
   * advanced tab
   */
  advanced = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (advanced);

  /* lbackup */
  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);

  image =
    gtk_image_new_from_icon_name ("document-save",
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 2);
  gtk_widget_show (image);

  label = gtk_label_new (_("Backup/Restore your system"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  backup_btn = gtk_button_new ();
  gtk_widget_set_sensitive (backup_btn,
			    (g_access ("/usr/bin/lbackup", X_OK) == 0));
  gtk_container_add (GTK_CONTAINER (backup_btn), hbox);
  gtk_widget_show (backup_btn);

  frame = xfce_framebox_new (_("Backups"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), backup_btn);
  gtk_box_pack_start (GTK_BOX (advanced), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /* persistence */
  vbox2 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);

  dialog->home_chk =
    gtk_radio_button_new_with_label (NULL,
				     (_("user settings only persistence")));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->home_chk, FALSE, FALSE, 0);
  gtk_widget_show (dialog->home_chk);
  dialog->full_chk =
    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
						 (dialog->home_chk),
						 _("full persistent system"));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->full_chk, FALSE, FALSE, 0);
  gtk_widget_show (dialog->full_chk);

  frame = xfce_framebox_new (_("Data persistence:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
  gtk_box_pack_start (GTK_BOX (advanced), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /* misc */
  vbox2 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);

  label = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label),
			_
			("<span style=\"italic\" size=\"small\">WARNING: Be sure to know "
			 "what you do before changing these settings.</span>"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 4);
  gtk_widget_show (label);

  dialog->nox_chk =
    gtk_check_button_new_with_label (_
				     ("Don't start Xorg (command line system)"));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->nox_chk, FALSE, FALSE, 0);
  gtk_widget_show (dialog->nox_chk);

  dialog->checkfs_chk =
    gtk_check_button_new_with_label (_
				     ("Check and try to fix filesystem errors"));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->checkfs_chk, FALSE, FALSE, 0);
  gtk_widget_show (dialog->checkfs_chk);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);

  image =
    gtk_image_new_from_icon_name ("edit-clear",
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 2);
  gtk_widget_show (image);

  label = gtk_label_new (_("Format an USB key (FAT32)"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  format_btn = gtk_button_new ();
  gtk_widget_set_sensitive (format_btn,
			    (g_access ("/usr/bin/lusbclean", X_OK) == 0));
  gtk_container_add (GTK_CONTAINER (format_btn), hbox);
  gtk_box_pack_start (GTK_BOX (vbox2), format_btn, FALSE, FALSE, 2);
  gtk_widget_show (format_btn);

  frame = xfce_framebox_new (_("Misc:"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
  gtk_box_pack_start (GTK_BOX (advanced), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  /*
   * soft lock tab
   */
  label = gtk_label_new (_("Advanced"));
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), advanced, label);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);

  dialog->lock_chk = gtk_check_button_new_with_label (_("Use the Linutop lock"));
  gtk_box_pack_start (GTK_BOX (vbox2), dialog->lock_chk, FALSE, FALSE, 4);
  gtk_widget_show (dialog->lock_chk);

  hr = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox2), hr, FALSE, FALSE, 2);
  gtk_widget_show (hr);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 2);
  gtk_widget_show (hbox);

  image = gtk_image_new_from_icon_name ("linutop-lock",
                                    GTK_ICON_SIZE_DIALOG);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 2);
  gtk_widget_show (image);

  label = gtk_label_new ("");
  gtk_label_set_markup (GTK_LABEL (label), 
    _("<span size=\"small\">"
      "In locked mode the system cannot be altered\n(convenient for public access).\n\n"
      "When switching to this mode, you will be prompted a password for the "
      "admin user (called 'linutop'). This user only can switch back to the "
      "standard mode. "
      "To change the login, use the 'logout' button in the quit dialog.\n\n"
      "To configure the unprivileged user, log in as 'user' in persistant mode. "
      "Regular persistent mode allows you to add softwares.\n\n"
      "Don't forget your password!"
      "</span>"));
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4);
  gtk_widget_show (label);

  hr = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox2), hr, FALSE, FALSE, 2);
  gtk_widget_show (hr);

  dialog->vbox_lock = gtk_vbox_new (FALSE, 2);
  gtk_widget_set_sensitive (dialog->vbox_lock, FALSE);
  gtk_widget_show (dialog->vbox_lock);

  dialog->lock_pwd_btn = gtk_button_new_with_label (_("Change password"));
  gtk_box_pack_start (GTK_BOX (dialog->vbox_lock), dialog->lock_pwd_btn, FALSE,
		      FALSE, 2);
  gtk_widget_show (dialog->lock_pwd_btn);

  lock_w = lock_widget ();
  gtk_box_pack_start (GTK_BOX (dialog->vbox_lock), lock_w, FALSE, FALSE, 2);
  gtk_widget_show (lock_w);

  frame = xfce_framebox_new (_("Settings"), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  xfce_framebox_add (XFCE_FRAMEBOX (frame), dialog->vbox_lock);
  gtk_box_pack_start (GTK_BOX (vbox2), frame, FALSE, FALSE, 10);
  gtk_widget_show (frame);

  label = gtk_label_new (_("Security"));
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox2, label);

  /*
   * view tab
   */
#ifdef HAVE_VIEW
  label = gtk_label_new (_("View"));
  scwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scwindow),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scwindow, label);
  gtk_widget_show (scwindow);

  dialog->text = gtk_text_view_new ();
  gtk_text_view_set_editable (GTK_TEXT_VIEW (dialog->text), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (dialog->text), FALSE);
  gtk_container_add (GTK_CONTAINER (scwindow), dialog->text);
  gtk_widget_show (dialog->text);
#endif /* HAVE_VIEW */

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->dialog)->action_area),
		     hbox);
  gtk_widget_show (hbox);

  dialog->help_btn = gtk_button_new_from_stock (GTK_STOCK_HELP);
  gtk_box_pack_start (GTK_BOX (hbox), dialog->help_btn, TRUE, FALSE, 0);
  gtk_widget_set_sensitive (dialog->help_btn, (g_access (README, R_OK) == 0));
  gtk_widget_show (dialog->help_btn);

  dialog->save_btn = gtk_button_new_from_stock (GTK_STOCK_OK);
  gtk_box_pack_end (GTK_BOX (hbox), dialog->save_btn, TRUE, FALSE, 0);
  gtk_widget_set_sensitive (dialog->save_btn,
			    (g_access (conf_file, W_OK) == 0));
  gtk_widget_show (dialog->save_btn);

  dialog->quit_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
  gtk_box_pack_end (GTK_BOX (hbox), dialog->quit_btn, TRUE, FALSE, 0);
  gtk_widget_show (dialog->quit_btn);

  read_config ();
  update_text (NULL, NULL);

  /* signals */
  g_signal_connect (G_OBJECT (dialog->tz_entry), "changed",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->locale_entry), "changed",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->kbd_entry), "changed",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->net_btn), "clicked",
		    G_CALLBACK (net_admin), NULL);
  g_signal_connect (G_OBJECT (dialog->screen_combo), "changed",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (asrun_btn), "clicked",
		    G_CALLBACK (appstarter), NULL);

  g_signal_connect (G_OBJECT (dialog->kbd_btn), "clicked",
		    G_CALLBACK (kbd_btn_clicked), NULL);
  g_signal_connect (G_OBJECT (dialog->tz_btn), "clicked",
		    G_CALLBACK (tz_btn_clicked), NULL);
  g_signal_connect (G_OBJECT (dialog->locale_btn), "clicked",
		    G_CALLBACK (locale_btn_clicked), NULL);

  g_signal_connect (G_OBJECT (dialog->full_chk), "toggled",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->home_chk), "toggled",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->nox_chk), "toggled",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (dialog->checkfs_chk), "toggled",
		    G_CALLBACK (update_text), NULL);
  g_signal_connect (G_OBJECT (backup_btn), "clicked",
		    G_CALLBACK (lbackup), NULL);
  g_signal_connect (G_OBJECT (format_btn), "clicked",
		    G_CALLBACK (usb_format), NULL);

  g_signal_connect (G_OBJECT (dialog->lock_chk), "toggled",
		    G_CALLBACK (lock_toggled), NULL);
  g_signal_connect (G_OBJECT (dialog->lock_pwd_btn), "clicked",
		    G_CALLBACK (change_pwd), NULL);

  g_signal_connect (G_OBJECT (dialog->help_btn), "clicked",
		    G_CALLBACK (run_help), NULL);
  g_signal_connect (G_OBJECT (dialog->save_btn), "clicked",
		    G_CALLBACK (save_config), NULL);
  g_signal_connect (G_OBJECT (dialog->quit_btn), "clicked",
		    G_CALLBACK (delete_event), NULL);
  g_signal_connect (G_OBJECT (dialog->dialog), "delete_event",
		    G_CALLBACK (delete_event), NULL);

  gtk_widget_show (dialog->dialog);
}

int
main (int argc, char *argv[])
{
  xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
  gtk_init (&argc, &argv);
  gdk_notify_startup_complete ();
  int id = getuid ();
  if (id != 0)
    {
      xfce_warn (_("You don't appear to be root. Exiting."));
      exit (1);
    }

  dsp = XOpenDisplay (NULL);
  g_type_init ();
  engine = xkl_engine_get_instance (dsp);
  registry = xkl_config_registry_get_instance (engine);
  xkl_config_registry_load (registry);

  conf_file = "/cdrom/syslinux.cfg";
  dialog = g_new0 (gui, 1);
  dialog->lock_pwd = NULL;
  alltrees = g_new0 (trees, 1);

  create_dialog ();

  gtk_main ();

//  gtk_widget_destroy (dialog->dialog);
  g_free (dialog);
  g_free (alltrees);
  g_free (initial_conf);

  if (reboot_needed)
      system (REBOOTCMD);

  return 0;
}
