/* lusbclean.c
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of lbackup
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <glib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#define MARGIN 2

GList *get_valid_devices();

static gboolean thread_ended = FALSE;
static GMutex *mutex = NULL;

static void
append_to_store(gpointer data, gpointer user_data)
{
    GtkListStore *store = (GtkListStore *)user_data;
    gchar        *device = (gchar *)data;
    GtkTreeIter   iter;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, device, -1);
}

static void
populate_store (GtkListStore *store)
{
    GList *devices = NULL;

    devices = get_valid_devices();
    gtk_list_store_clear (store);
    g_list_foreach (devices, (GFunc) append_to_store, store);
    g_list_free (devices);
}

static void
reload_clicked (GtkWidget *btn, gpointer data)
{
    GtkWidget *combo = (GtkWidget *)data;
    GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
    populate_store (GTK_LIST_STORE (model));
    gtk_combo_box_set_active(GTK_COMBO_BOX (combo), 0);
}

static void
combo_changed (GtkWidget *combo, gpointer data)
{
    gtk_widget_set_sensitive ((GtkWidget *) data,
        (gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) >= 0) ? TRUE : FALSE);
}

gboolean
check_thread (gpointer data)
{
    g_mutex_lock (mutex);
    if (thread_ended)
    {
        GtkWidget *d = (GtkWidget *) data;
        gtk_widget_destroy (d);
        g_mutex_unlock (mutex);
        g_mutex_free (mutex);
        xfce_info ("You device is clean.");
        return FALSE;
    }
    g_mutex_unlock (mutex);
    return TRUE;
}

gpointer
run_script (gpointer data)
{
    gchar *cmd = (gchar *)data;
    system (cmd);
    g_mutex_lock (mutex);
    thread_ended = TRUE;
    g_mutex_unlock (mutex);
    return (gpointer) TRUE;
}

static void
go_clicked (GtkWidget *btn, gpointer data)
{
    GtkWidget    *combo = (GtkWidget *) data;
    GtkTreeIter   iter;
    GtkTreeModel *model;
    gchar        *device = NULL;
    gint          response;
    gchar         txt[95];

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter);
    gtk_tree_model_get (model, &iter, 0, &device, -1);

    g_snprintf (txt, 64 + strlen(device),
                     _("The content of %s will be erased.\n"
                       "Do you really want to continue?"),
                     device);
    response = xfce_confirm (txt, GTK_STOCK_OK, NULL);
    if (!response)
    {
        g_free (device);
        return;
    }

    GThread *thread;
    GtkWidget *d;
    mutex = g_mutex_new ();
    g_snprintf (txt, 15 + strlen (device), "lusm-clean -y %s", device);
    thread = g_thread_create ((GThreadFunc) run_script,
                               txt,
                               FALSE,
                               NULL);
    d = xfce_message_dialog_new (NULL,
                                 _("Formatting"),
                                 GTK_STOCK_DIALOG_INFO,
                                 _("Formatting"),
                                 _("You device is being prepared.\n"
                                   "Please wait..."),
                                 NULL);
    gtk_window_set_deletable (GTK_WINDOW (d), FALSE);
    g_timeout_add (1000, check_thread, (gpointer) d);
    gtk_dialog_run (GTK_DIALOG (d));
    g_free (device);
}

static void
build_gui()
{
    GtkWidget       *w, *label, *go_btn, *quit_btn, *reload_btn;
    GtkWidget       *combo;
    GtkWidget       *vbox, *hbox;
    GtkWidget       *heading;
    GtkListStore    *store;
    GtkCellRenderer *renderer;

    gtk_window_set_default_icon_name ("usbpendrive_mount");
    w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (w), _("USB Key"));
    gtk_window_set_resizable (GTK_WINDOW (w), FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (w), 6);
    vbox = gtk_vbox_new (FALSE, MARGIN);
    gtk_container_add(GTK_CONTAINER (w), vbox);
    heading = xfce_heading_new();
    xfce_heading_set_title(XFCE_HEADING (heading), _("Linutop USB cleaner"));
    xfce_heading_set_subtitle(XFCE_HEADING (heading),
        _("Format an USB key"));
    xfce_heading_set_icon_name (XFCE_HEADING (heading), "usbpendrive_mount");
    gtk_box_pack_start(GTK_BOX (vbox), heading, FALSE, FALSE, MARGIN);
    hbox = gtk_hbox_new (FALSE, MARGIN);
    gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, MARGIN);
    label = gtk_label_new (_("Device: "));
    gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, MARGIN);

    store = gtk_list_store_new (1, G_TYPE_STRING);
    combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL (store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (
        GTK_CELL_LAYOUT (combo), renderer,  "text", 0);
    gtk_box_pack_start(GTK_BOX (hbox), combo, TRUE, TRUE, MARGIN);
    reload_btn = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
    gtk_box_pack_start(GTK_BOX (hbox), reload_btn, FALSE, FALSE, MARGIN);

    label = gtk_label_new ("");
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span style=\"italic\" size=\"small\">Click the 'refresh' button "
        "to detect newly plugged devices</span>"));
    gtk_box_pack_start(GTK_BOX (vbox), label, FALSE, FALSE, MARGIN);
    gtk_widget_show (label);

    hbox = gtk_hbox_new (FALSE, MARGIN);
    gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, MARGIN);
    quit_btn = gtk_button_new_from_stock (GTK_STOCK_QUIT);
    gtk_box_pack_end(GTK_BOX (hbox), quit_btn, FALSE, FALSE, MARGIN);
    go_btn = gtk_button_new_from_stock (GTK_STOCK_APPLY);
    gtk_box_pack_end(GTK_BOX (hbox), go_btn, FALSE, FALSE, MARGIN);

    g_signal_connect (G_OBJECT (w), "destroy", gtk_main_quit, NULL);
    g_signal_connect (G_OBJECT (quit_btn), "clicked", gtk_main_quit, NULL);
    g_signal_connect (G_OBJECT (reload_btn), "clicked",
        G_CALLBACK (reload_clicked), combo);
    g_signal_connect (G_OBJECT (combo), "changed",
        G_CALLBACK (combo_changed), go_btn);
    g_signal_connect (G_OBJECT (go_btn), "clicked",
        G_CALLBACK (go_clicked), combo);

    populate_store(store);
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);

    gtk_widget_show_all (w);
}

int main(int argc, char *argv[])
{
    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    g_thread_init (NULL);
    gtk_init(&argc, &argv);

    if (getuid() != 0)
    {
        xfce_warn(_("You don't appear to be root. Exiting."));
        exit (1);
    }

    build_gui();
    gtk_main();

    return 0;
}
